/*  Copyright (C) 2023 Tibor 'Igor2' Palinkas - license: Creative Commons CC0 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include <libgpcogl/libgpcogl.h>
#include <libgpcogl/gli.h>

#define NL "\n"

const char *dist_sq =
	NL GPCOGL_SHADER_PROG(130)                /* program header, require glsl ver 1.3 */
	NL GPCOGL_OUT_1_UINT32(output_cell)       /* specify output cell type (1*uint32), create a variable output_cell for the output */
	NL "void main()"                          /* shader program entry point */
	NL "{"
	NL "	" GPCOGL_OUT_COORDS(x, y)           /* fetch output cell coords into variables x and y */
	NL "	output_cell = uint(x*x+y*y);"       /* the actual calculation - distance square from 0;0; need to convert to uint because x and y are int but outpout is uint */
	NL "}";

static void print_mx(uint32_t *f, int sx, int sy)
{
	int x, y;

	for(y = 0; y < sy; y++) {
		for(x = 0; x < sx; x++) {
			if (x > 0)
				printf(" ");
			printf("% 4d", *f);
			f++;
		}
		printf("\n");
	}
}

int main(int argc, char **argv)
{
	int sx = 20, sy = 20; /* size of our output array */
	gpcogl_prog_id_t prg; /* handle of the shader program */
	gpcogl_t ctx;         /* lib context all arrays and shader programs are in */
	gpcogl_array_id_t ot; /* handle of the output array (GPU-side) */
	uint32_t *res;        /* CPU-side copy of the output array */

	/* Initialize the opengl interface library (e.g. glut or noxgl) */
	gpcogl_gli_init(&argc, argv);

	/* Create a context (hidden X window with glut) for off-screen rendering */
	if (gpcogl_gli_make_context(&ctx, NULL) != 0)
		return 1;

	/* Compile the shader program from dist_sq, remember it as prg */
	prg = gpcogl_compile(&ctx, dist_sq);
	if (prg == 0) {
		fprintf(stderr, "Failed to compile dist_sq\n");
		return 1;
	}

	/* create a GPU array (would be the output of compute) */
	ot = gpcogl_array_create(&ctx, GPCOGL_CT_1_UINT32, sx, sy);

	/* Compute: use precompiled program "prg", output to "ot", no input */
	gpcogl_vcompute(&ctx, prg, ot, NULL);

	/* Allocate memory in CPU ram for the result and download from "ot" to res */
	res = calloc(sx*sy, sizeof(uint32_t));
	gpcogl_array_get(&ctx, res, ot);

	/* print result */
	print_mx(res, sx, sy);

	/* free context and lib globals */
	gpcogl_gli_destroy_context(&ctx);
	gpcogl_gli_uninit();

	return 0;
}
