/*
    libgpcogl - Generic Purpose Computation on OpenGL (using shaders)
    Copyright (C) 2023 Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Contact:
      Project page: http://www.repo.hu/projects/libgpcogl
      Lead developer: http://www.repo.hu/projects/libgpcogl/contact.html
*/


#include <stdlib.h>
#include <libnoxgl.h>

#include "gli_noxgl.h"

static int switch_context_noxgl(gpcogl_t *ctx)
{
	noxgl_make_current_context(ctx->ogl_ctx.p0, ctx->ogl_ctx.p1, ctx->ogl_ctx.p2);
	return 0;
}

int gpcogl_noxgl_make_context(gpcogl_t *ctx, const char *device)
{
	ctx->ogl_ctx.p0 = noxgl_init(device, 1);

	if (ctx->ogl_ctx.p0 == NULL)
		return -1;

	if (noxgl_create_context(ctx->ogl_ctx.p0, &ctx->ogl_ctx.p1, &ctx->ogl_ctx.p2) != 0)
		return -1;

	ctx->make_current = switch_context_noxgl;
	if (gpcogl_init_context(ctx) != 0)
		return -1;

	gpcogl_redir_render_to_fb(ctx);

	return 0;
}

void gpcogl_noxgl_destroy_context(gpcogl_t *ctx)
{
	gpcogl_uninit_context(ctx);
	noxgl_destroy_context(ctx->ogl_ctx.p0, ctx->ogl_ctx.p1, ctx->ogl_ctx.p2);
	noxgl_uninit(ctx->ogl_ctx.p0);

	ctx->ogl_ctx.p0 = ctx->ogl_ctx.p1 = ctx->ogl_ctx.p2 = NULL;
}

#ifndef MULTIPLE_GLIS

#include <stdio.h>
#include "gli.h"

void gpcogl_gli_init(int *argc, char *argv[])
{
}

void gpcogl_gli_uninit(void)
{
}

int gpcogl_gli_make_context(gpcogl_t *ctx, const char *option)
{
	if (option == NULL)
		option = getenv("GPCOGL_NOXGL_DEVICE");
	if (option == NULL)
		option = "/dev/dri/card0";
	return gpcogl_noxgl_make_context(ctx, option);
}

int gpcogl_gli_make_context_window(gpcogl_t *ctx, int wsx, int wsy)
{
	fprintf(stderr, "gpcogl_gli_make_context_window() is not supported with noxgl\n");
	abort();
}

void gpcogl_gli_destroy_context(gpcogl_t *ctx)
{
	gpcogl_noxgl_destroy_context(ctx);
}

#endif

